#include <OneWire.h>
#include <DallasTemperature.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include "Ultrasonic.h"

#define TEMP_SENSOR 2
#define SHARP_IR_PIN A0
#define MOTOR 9

OneWire oneWire(TEMP_SENSOR);
DallasTemperature sensors(&oneWire);
LiquidCrystal_I2C lcd(0x27, 16, 4);
Ultrasonic ultrasonic(7, 6);

void setup() {
  pinMode(MOTOR, OUTPUT);
  lcd.init();
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.print("    ARDUINO     ");
  lcd.setCursor(0, 1);
  lcd.print("TEMPRETURE METER");
  sensors.begin();
  Serial.begin(9600);
}

float factor = 0;
int motorTuration = 0;
bool isFirstTime = true;

void printOnLCD(float temperatureC, float temperatureF, float motorTuration, float dist)
{
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Temp in C = ");
  lcd.print(temperatureC);
  lcd.print((char)223);
  lcd.print("C");
  lcd.setCursor(0, 2);
  lcd.print("Motor turation = ");
  lcd.print(motorTuration);
  lcd.setCursor(0, 3);
  lcd.print("Distance = ");
  lcd.print(dist);
  
}
//float temperatureC = 22.94;
//float temperatureF = 67.32;
void loop() {

  sensors.requestTemperatures();
  float temperatureC = sensors.getTempCByIndex(0);
  float temperatureF = sensors.getTempFByIndex(0);

  Serial.print("motor turation =");
  Serial.println(motorTuration);

  long rangeInCentimeters = ultrasonic.read();
  Serial.print("RangeInCentimeters = ");
  Serial.println(rangeInCentimeters);
    if (motorTuration == 0) {
      if (temperatureC >= 21) {
        motorTuration = 140;
        analogWrite(MOTOR, motorTuration);
        Serial.println(motorTuration);
        delay(4000);
        return; 
      }
      
    }
    if (rangeInCentimeters > 40 && isFirstTime == true) {
      if (temperatureC < 25) {
        if (temperatureC >= 24) {
          factor = (int)105 * 13 / temperatureC;
          motorTuration = 105;
        }
        else if (temperatureC >= 23 && temperatureC < 24){
          factor = 58;
          motorTuration = 102;
        }
        else if (temperatureC >= 22 && temperatureC < 23){
          factor = 58;
          motorTuration = 101;
        }
        else if (temperatureC >= 21 && temperatureC < 22)
        {
          if (temperatureC >= 21 && temperatureC <= 21.3){
            factor = 62;
          } else if (temperatureC > 21.3 && temperatureC <= 21.5){
            factor = 61;
          } else {
            factor = 60;
            
          }
          motorTuration = (int)(temperatureC * factor / 13);
        }
        Serial.println(motorTuration);
        analogWrite(MOTOR, 100);
        delay(6000);
        isFirstTime = false;
        return;
      }
    }

  if (rangeInCentimeters == 37) {
    if (temperatureC >= 21) {
      factor += 0.5;
    }
  }
  if (temperatureC < 21.0) {
    motorTuration = 0;
    analogWrite(MOTOR, motorTuration);
    isFirstTime = true;
  }
  else if (temperatureC > 25.0) {
    motorTuration = 255;
    analogWrite(MOTOR, 255); 
  }
  else if (temperatureC < 22.0 && temperatureC >= 21.0) {
    if (rangeInCentimeters >= 33) {
      if (motorTuration >= 104){
        printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
        return;
      }
      factor = factor + 0.1;
    } else {
      if (rangeInCentimeters < 28) {
        factor = factor -  0.1;
        printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
        motorTuration = (int)(temperatureC * factor / 13);
        analogWrite(MOTOR, motorTuration);
        return;
      }
        
      if (motorTuration <= 102){
        printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
        return;
      }
      factor = factor -  0.1;
    } 
    motorTuration = (int)(temperatureC * factor / 13);
    analogWrite(MOTOR, motorTuration);
  } else if (temperatureC < 23.0 && temperatureC >= 22.0) {
    if (rangeInCentimeters > 25) {
      if (motorTuration >= 104){
        printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
        return;
      }
      factor = factor + 0.1;
    } else {//if (rangeInCentimeters < 27){
      if (motorTuration <= 101) {
        if (rangeInCentimeters < 18) {
          
          factor = factor -  0.1;
          motorTuration = (int)(temperatureC * factor / 13);
          analogWrite(MOTOR, motorTuration);
        }
        return;
      }
      if (motorTuration <= 103){
        if (rangeInCentimeters < 18) {
          printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
          factor = factor -  0.1;
          motorTuration = (int)(temperatureC * factor / 13);
          analogWrite(MOTOR, motorTuration);
        }
        return;
      }
      factor = factor -  0.1;
    } 
    motorTuration = (int)(temperatureC * factor / 13);
    analogWrite(MOTOR, motorTuration);
  } else if (temperatureC < 24.0 && temperatureC >= 23.0) {
    if (rangeInCentimeters > 19) {
      if (motorTuration >= 104) {
        printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
        return;
      }
      factor = factor + 0.1;
    } else {//if (rangeInCentimeters < 27){
      if (rangeInCentimeters < 9) {
        factor = factor -  0.2;
        motorTuration = (int)(temperatureC * factor / 13);
        analogWrite(MOTOR, motorTuration);
        return;
      }
      if (motorTuration <= 103) {
        printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
        return;
      }
      printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
      factor = factor -  0.2;
    } 
    motorTuration = (int)(temperatureC * factor / 13);
    analogWrite(MOTOR, motorTuration);
  } else {
    if (rangeInCentimeters > 9 && rangeInCentimeters != 42 && rangeInCentimeters != 41) {
      if (motorTuration >= 106) {
        printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);;
        return;
      }
      factor = factor + 0.1;
    } else {//if (rangeInCentimeters < 27){
      if (rangeInCentimeters <= 4 || rangeInCentimeters >= 41) {
        if (motorTuration < 103) {
          printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
          return;
        }
        factor = factor -  0.15;
        motorTuration = (int)(temperatureC * factor / 13);
        analogWrite(MOTOR, motorTuration);
        printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
        return;
      }

      if (motorTuration <= 104){
        printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
        return;
      }
      factor = factor -  0.15;
    } 
    motorTuration = (int)(temperatureC * factor / 13);
    analogWrite(MOTOR, motorTuration);
  }
  printOnLCD(temperatureC, temperatureF, motorTuration, rangeInCentimeters);
}
